@@ -18,7 +18,7 @@
 */
 
 #include <ql/qldefines.hpp>
-#ifdef BOOST_MSVC
+#if !defined(BOOST_ALL_NO_LIB) && defined(BOOST_MSVC)
 #  include <ql/auto_link.hpp>
 #endif
 #include <ql/instruments/floatfloatswap.hpp>
@@ -89,7 +89,7 @@ void printBasket(
     }
 }
 
-// helper function that prints the result of a model calibraiton to std::cout
+// helper function that prints the result of a model calibration to std::cout
 
 void printModelCalibration(
     const std::vector<ext::shared_ptr<BlackCalibrationHelper> > &basket,
