@@ -33,8 +33,11 @@ namespace QuantLib {
 
     // functions
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T, class U>
-    class constant {
+    class QL_DEPRECATED constant {
       public:
         typedef T argument_type;
         typedef U result_type;
@@ -44,8 +47,11 @@ namespace QuantLib {
         U u_;
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class identity {
+    class QL_DEPRECATED identity {
       public:
         typedef T argument_type;
         typedef T result_type;
@@ -53,23 +59,35 @@ namespace QuantLib {
     };
 
     template <class T>
-    class square {
+    inline T squared(T x) { return x * x; }
+
+    /*! \deprecated Use squared or a lambda instead.
+                    Deprecated in version 1.27.
+    */
+    template <class T>
+    class QL_DEPRECATED square {
       public:
         typedef T argument_type;
         typedef T result_type;
         T operator()(const T& t) const { return t*t; }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class cube {
+    class QL_DEPRECATED cube {
       public:
         typedef T argument_type;
         typedef T result_type;
         T operator()(const T& t) const { return t*t*t; }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class fourth_power {
+    class QL_DEPRECATED fourth_power {
       public:
         typedef T argument_type;
         typedef T result_type;
@@ -78,143 +96,178 @@ namespace QuantLib {
 
     // a few shortcuts for common binders
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class add {
+    class QL_DEPRECATED add {
         T y;
       public:
         typedef T argument_type;
         typedef Real result_type;
-
         explicit add(Real y) : y(y) {}
         Real operator()(T x) const { return x + y; }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class subtract {
+    class QL_DEPRECATED subtract {
         T y;
       public:
         typedef T argument_type;
         typedef Real result_type;
-
         explicit subtract(Real y) : y(y) {}
         Real operator()(T x) const { return x - y; }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class subtract_from {
+    class QL_DEPRECATED subtract_from {
         T y;
       public:
         typedef T argument_type;
         typedef Real result_type;
-
         explicit subtract_from(Real y) : y(y) {}
         Real operator()(T x) const { return y - x; }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class multiply_by {
+    class QL_DEPRECATED multiply_by {
         T y;
       public:
         typedef T argument_type;
         typedef Real result_type;
-
         explicit multiply_by(Real y) : y(y) {}
         Real operator()(T x) const { return x * y; }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class divide {
+    class QL_DEPRECATED divide {
         T y;
       public:
         typedef T argument_type;
         typedef Real result_type;
-
         explicit divide(Real y) : y(y) {}
         Real operator()(T x) const { return y / x; }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class divide_by {
+    class QL_DEPRECATED divide_by {
         T y;
       public:
         typedef T argument_type;
         typedef Real result_type;
-
         explicit divide_by(Real y) : y(y) {}
         Real operator()(T x) const { return x / y; }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class less_than {
+    class QL_DEPRECATED less_than {
         T y;
       public:
         typedef T argument_type;
         typedef bool result_type;
-
         explicit less_than(Real y) : y(y) {}
         bool operator()(T x) const { return x < y; }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class greater_than {
+    class QL_DEPRECATED greater_than {
         T y;
       public:
         typedef T argument_type;
         typedef bool result_type;
-
         explicit greater_than(Real y) : y(y) {}
         bool operator()(T x) const { return x > y; }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class greater_or_equal_to {
+    class QL_DEPRECATED greater_or_equal_to {
         T y;
       public:
         typedef T argument_type;
         typedef bool result_type;
-
         explicit greater_or_equal_to(Real y) : y(y) {}
         bool operator()(T x) const { return x >= y; }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class not_zero {
+    class QL_DEPRECATED not_zero {
       public:
         typedef T argument_type;
         typedef bool result_type;
         bool operator()(T x) const { return x != T(); }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class not_null {
+    class QL_DEPRECATED not_null {
         T null;
       public:
         typedef T argument_type;
         typedef bool result_type;
-
         not_null() : null(Null<T>()) {}
         bool operator()(T x) const { return x != null; }
     };
     
     // predicates
 
-    class everywhere : public constant<Real,bool> {
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
+    class QL_DEPRECATED everywhere {
       public:
-        everywhere() : constant<Real,bool>(true) {}
+        typedef Real argument_type;
+        typedef bool result_type;
+        bool operator()(Real) const { return true; }
     };
 
-    class nowhere : public constant<Real,bool> {
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
+    class QL_DEPRECATED nowhere {
       public:
-        nowhere() : constant<Real,bool>(false) {}
+        typedef Real argument_type;
+        typedef bool result_type;
+        bool operator()(Real) const { return false; }
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class T>
-    class equal_within {
+    class QL_DEPRECATED equal_within {
       public:
         typedef T first_argument_type;
         typedef T second_argument_type;
         typedef bool result_type;
-
         explicit equal_within(const T& eps) : eps_(eps) {}
         bool operator()(const T& a, const T& b) const {
             return std::fabs(a-b) <= eps_;
@@ -224,8 +277,12 @@ namespace QuantLib {
     };
 
     // combinators
+
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class F, class R>
-    class clipped_function {
+    class QL_DEPRECATED clipped_function {
       public:
         typedef typename F::argument_type argument_type;
         typedef typename F::result_type result_type;
@@ -238,20 +295,30 @@ namespace QuantLib {
         R r_;
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
 #if __cplusplus >= 201402L || _MSVC_LANG >= 201402L
     template <class F, class R>
+    QL_DEPRECATED
     auto clip(F&& f, R&& r) {
         return [f_ = std::forward<F>(f), r_ = std::forward<R>(r)](const auto& x) { return r_(x) ? f_(x) : decltype(f_(x)){}; };
     }
 #else
+    QL_DEPRECATED_DISABLE_WARNING
     template <class F, class R>
+    QL_DEPRECATED
     clipped_function<F,R> clip(const F& f, const R& r) {
         return clipped_function<F,R>(f,r);
     }
+    QL_DEPRECATED_ENABLE_WARNING
 #endif
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class F, class G>
-    class composed_function {
+    class QL_DEPRECATED composed_function {
       public:
         typedef typename G::argument_type argument_type;
         typedef typename F::result_type result_type;
@@ -264,49 +331,63 @@ namespace QuantLib {
         G g_;
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
 #if __cplusplus >= 201402L || _MSVC_LANG >= 201402L
     template <class F, class G>
+    QL_DEPRECATED
     auto compose(F&& f, G&& g) {
         return [f_ = std::forward<F>(f), g_ = std::forward<G>(g)](const auto& x) { return f_(g_(x)); };
     }
 #else
+    QL_DEPRECATED_DISABLE_WARNING
     template <class F, class G>
+    QL_DEPRECATED
     composed_function<F,G> compose(const F& f, const G& g) {
         return composed_function<F,G>(f,g);
     }
+    QL_DEPRECATED_ENABLE_WARNING
 #endif
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
     template <class F, class G, class H>
-    class binary_compose3_function {
+    class QL_DEPRECATED binary_compose3_function {
       public:
         typedef typename G::argument_type first_argument_type;
         typedef typename H::argument_type second_argument_type;
         typedef typename F::result_type result_type;
-
         binary_compose3_function(const F& f, const G& g, const H& h)
         : f_(f), g_(g), h_(h) {}
-
         result_type operator()(const first_argument_type&  x,
                                const second_argument_type& y) const {
             return f_(g_(x), h_(y));
         }
-
       private:
         F f_;
         G g_;
         H h_;
     };
 
+    /*! \deprecated Use a lambda instead.
+                    Deprecated in version 1.27.
+    */
 #if __cplusplus >= 201402L || _MSVC_LANG >= 201402L
     template <class F, class G, class H>
+    QL_DEPRECATED
     auto compose3(F&& f, G&& g, H&& h) {
         return [f_ = std::forward<F>(f), g_ = std::forward<G>(g), h_ = std::forward<H>(h)](const auto& x, const auto& y) { return f_(g_(x), h_(y)); };
     }
 #else
+    QL_DEPRECATED_DISABLE_WARNING
     template <class F, class G, class H> binary_compose3_function<F, G, H>
+    QL_DEPRECATED
     compose3(const F& f, const G& g, const H& h) {
         return binary_compose3_function<F, G, H>(f, g, h);
     }
+    QL_DEPRECATED_ENABLE_WARNING
 #endif
 }
 
