@@ -116,8 +116,10 @@ namespace QuantLib {
 
         ext::shared_ptr<ZeroInflationIndex> new_zii = zii_->clone(zits);
 
+        QL_DEPRECATED_DISABLE_WARNING
         Handle<YieldTermStructure> nominalTS =
             !nominalTermStructure_.empty() ? nominalTermStructure_ : z->nominalTermStructure();
+        QL_DEPRECATED_ENABLE_WARNING
 
         Real nominal = 1000000.0; // has to be something but doesn't matter what
         Date start = nominalTS->referenceDate();
@@ -223,8 +225,11 @@ namespace QuantLib {
 
         // The instrument takes a standard discounting swap engine.
         // The inflation-related work is done by the coupons.
+        QL_DEPRECATED_DISABLE_WARNING
         Handle<YieldTermStructure> nominalTS =
             !nominalTermStructure_.empty() ? nominalTermStructure_ : y->nominalTermStructure();
+        QL_DEPRECATED_ENABLE_WARNING
+
         yyiis_->setPricingEngine(
             ext::shared_ptr<PricingEngine>(new DiscountingSwapEngine(nominalTS)));
     }
